package com.jiayue.insu.incloud.pushdata;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.file.FileReader;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jiayue.insu.incloud.compermisson.BJZYYJYSDComPermisson;
import com.jiayue.insu.incloud.constants.CommonStant;
import com.jiayue.insu.incloud.constants.CompanyConstant;
import com.jiayue.insu.incloud.constants.enums.StatusEnum;
import com.jiayue.insu.incloud.constants.vo.ResponseLXVo;
import com.jiayue.insu.incloud.entity.ForecastDataLx;
import com.jiayue.insu.incloud.entity.IntegrationCompany;
import com.jiayue.insu.incloud.entity.Record;
import com.jiayue.insu.incloud.entity.Station;
import com.jiayue.insu.incloud.service.ForecastDataLxService;
import com.jiayue.insu.incloud.service.IntegrationCompanyService;
import com.jiayue.insu.incloud.service.RecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * 林祥上送短期数据
 *
 * @author yh
 * @version 1.0
 * @since 2022/5/22 11:29
 */
@Component
@Slf4j
public class PushDataForLX implements IPushInitForecastData {
    @Autowired
    private RecordService recordService;
    @Autowired
    private IntegrationCompanyService integrationCompanyService;
    @Autowired
    BJZYYJYSDComPermisson bjzyyjysdComPermisson;
    @Autowired
    private ForecastDataLxService forecastDataLxService;

    @Value("${localFilePath.lxFilePath}")
    String lxFilePath;

    /**
     * 推送原始预测数据
     *
     * @param station 场站信息
     * @return 是否成功
     */
    @Override
    public Boolean pushDQAndNWPData(Station station) {
        return null;
    }

    /**
     * 将下载好的原始短期文件通过sftp推送
     *
     * @param station
     * @return
     */
    @Override
    public Boolean pushDQData(Station station) {

        IntegrationCompany integrationCompany = integrationCompanyService.findByCode(CompanyConstant.COMPANY_LX);
        // 获取场站编号
        String stationCode = station.getStationCode();
        Record record = new Record(CommonStant.RECORD_TYPE_PUSH_INIT, CompanyConstant.COMPANY_LX, stationCode, null, LocalDateTimeUtil.beginOfDay(LocalDateTime.now()), LocalDateTime.now(), null, null);
        record.setState(StatusEnum.SUCCESS.getSign());
        // 如果场站编号为空，则返回
        if (stationCode == null || "".equals(stationCode)) {
            record.setState(StatusEnum.STATION_CODE_NULL.getSign());
            record.setStateContent(StatusEnum.STATION_CODE_NULL.getMsg());
            recordService.save(record);
            return false;
        }
        // 获取指定目录下的短期原始数据(/jiayue/insu/file/init/J00269/)
        String filePath = lxFilePath + stationCode + File.separatorChar;
        File files = new File(filePath);
        String[] fileNames = files.list();
        // 如果文件名列表为空
        if (fileNames == null || fileNames.length == 0) {
            log.info("指定目录下无文件");
            record.setState(StatusEnum.FILE_NULL.getSign());
            record.setStateContent(StatusEnum.FILE_NULL.getMsg());
            recordService.save(record);
            return false;
        }
        // 获取当天日期
        LocalDateTime today = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd");
        String todayStr = today.format(formatter);
        // 转为list并根据日期筛选排序
        List<String> fileList = Arrays.stream(fileNames).collect(Collectors.toList());
        List<String> filterFileList = fileList.stream().filter(s -> s.contains(todayStr)).filter(s -> s.contains("DQ")).collect(Collectors.toList());
        // 如果筛选后无文件
        if (filterFileList.size() == 0) {
            log.info("无当日文件");
            record.setState(StatusEnum.FILE_NULL.getSign());
            record.setStateContent(StatusEnum.FILE_NULL.getMsg());
            recordService.save(record);
            return false;
        }
        filterFileList.sort(Collections.reverseOrder());
        // 获取原始文件
        File originalFile = new File(filterFileList.get(0));
        // 读取文件
        FileReader fileReader = new FileReader(filePath + originalFile);
        List<String> originalContents = fileReader.readLines();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");

        String genDate = dateFormat.format(new Date());

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        List<Double> dataList = new ArrayList<>();

        StringBuilder content = new StringBuilder();
        content = content.append(stationCode + "-dq|" + dateFormat.format(new Date(System.currentTimeMillis() + 24 * 60 * 60 * 1000L)) + "|");

        List<ForecastDataLx> forecastDataLxList = new ArrayList<>();

        List<ForecastDataLx> complementList = new ArrayList<>();

        List<ForecastDataLx> goOnList = new ArrayList<>();

        int i = 0;
        // 循环解析原始短期文件
        for (String originalContent : originalContents) {
            // 林祥上送数据开始时间为当天的00:00:00
            if (originalContent.contains("#")) {
                // 根据分隔符分割，取时间和功率
                String[] originalContentArr = originalContent.split("\t");

                // 如果分割后的数组长度小于4，则数据格式不正确
                if (originalContentArr.length < 4) {
                    continue;
                }
                // 上送三天数据
                try {
                    if ("J00797".equals(stationCode) || "J00637".equals(stationCode)) {
                        if (95 < i && i < 1248) {
                            content.append(Double.valueOf(originalContentArr[3])).append("|");
                            ForecastDataLx forecastDataLx = new ForecastDataLx();
                            forecastDataLx.setGenDate(genDate);
                            forecastDataLx.setFpValue(new BigDecimal(originalContentArr[3]));
                            forecastDataLx.setForecastTime(format.parse(originalContentArr[2]).getTime());
                            forecastDataLx.setStationCode(stationCode);
                            forecastDataLx.setInCode(station.getInCode());
                            // 预测日期yyyy-MM-dd
                            forecastDataLx.setForecastDate(dateFormat.format(new Date(forecastDataLx.getForecastTime())));
                            forecastDataLxList.add(forecastDataLx);
                            dataList.add(Double.valueOf(originalContentArr[3]));
                        }
                    } else {
                        if (95 < i && i < 1056){
                            content.append(Double.valueOf(originalContentArr[3])).append("|");
                            dataList.add(Double.valueOf(originalContentArr[3]));
                        }
                    }


                } catch (Exception e) {
                    e.printStackTrace();
                    break;
                }
                i++;
            }
        }

        if ("J00797".equals(stationCode) || "J00637".equals(stationCode)) {
            // 一次补数
            forecastDataLxList.stream().sorted(Comparator.comparing(ForecastDataLx::getForecastTime)).collect(Collectors.toList());

            AtomicReference<String> changeForecastDate = new AtomicReference<>(addOneDay(forecastDataLxList.get(forecastDataLxList.size() - 1).getForecastDate()));
            // 分组
            Map<String, List<ForecastDataLx>> groupedByForecastDate = forecastDataLxList.stream()
                    .collect(Collectors.groupingBy(ForecastDataLx::getForecastDate));
            groupedByForecastDate.forEach((date, list) -> {
                String[] result = changeForecastDate.get().split("--");
                String genDateSpilt = result[0];

                Long forecastSpit = Long.valueOf(result[1]);
                AtomicInteger flag = new AtomicInteger();
                BigDecimal random = new BigDecimal(getRandom());
                list.forEach(forecastData -> {
                    ForecastDataLx forecastDataLx = new ForecastDataLx();
                    forecastDataLx.setFpValue(forecastData.getFpValue().multiply(random).setScale(2, RoundingMode.HALF_DOWN));
                    forecastDataLx.setForecastDate(genDateSpilt);
                    Long forecast = forecastSpit + flag.get() * 15 * 60 * 1000L;
                    forecastDataLx.setForecastTime(forecast);
                    forecastDataLx.setStationCode(stationCode);
                    forecastDataLx.setInCode(station.getInCode());
                    flag.getAndIncrement();
                    complementList.add(forecastDataLx);
                });
                changeForecastDate.set(addOneDay(genDateSpilt));

            });

            // 二次补数
            complementList.stream().sorted(Comparator.comparing(ForecastDataLx::getForecastTime)).collect(Collectors.toList());

            AtomicReference<String> changeComplementDate = new AtomicReference<>(addOneDay(complementList.get(complementList.size() - 1).getForecastDate()));
            // 分组
            Map<String, List<ForecastDataLx>> groupedByComplement = complementList.stream()
                    .collect(Collectors.groupingBy(ForecastDataLx::getForecastDate));
            final int[] a = {0};
            groupedByComplement.forEach((date, list) -> {
                String[] result = changeComplementDate.get().split("--");
                String genDateSpilt = result[0];

                Long forecastSpit = Long.valueOf(result[1]);
                BigDecimal random = new BigDecimal(getRandom());
                if (a[0] <= 5) {
                    AtomicInteger flag = new AtomicInteger();
                    list.forEach(forecastData -> {
                        ForecastDataLx forecastDataLx = new ForecastDataLx();
                        forecastDataLx.setFpValue(forecastData.getFpValue().multiply(random).setScale(2, RoundingMode.HALF_DOWN));
                        forecastDataLx.setForecastDate(genDateSpilt);
                        Long forecast = forecastSpit + flag.get() * 15 * 60 * 1000L;
                        forecastDataLx.setForecastTime(forecast);
                        forecastDataLx.setStationCode(stationCode);
                        forecastDataLx.setInCode(station.getInCode());
                        flag.getAndIncrement();
                        goOnList.add(forecastDataLx);
                    });
                }
                changeComplementDate.set(addOneDay(genDateSpilt));
                a[0]++;
            });
            complementList.addAll(goOnList);

            complementList.stream().sorted(Comparator.comparing(ForecastDataLx::getForecastTime)).collect(Collectors.toList());
            for (ForecastDataLx forecastDataLx : complementList) {
                content.append(forecastDataLx.getFpValue().doubleValue()).append("|");
            }
            forecastDataLxList.addAll(complementList);
        }

        String body = content.toString();
        List<String> stringList = new ArrayList<>();
        stringList.add(body);
        String jsonString = "";
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            // 将 List<String> 转换为 JSON 格式的字符串
            jsonString = objectMapper.writeValueAsString(stringList);
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            String response;
            HttpRequest httpRequest = HttpRequest.post("http://112.2.27.84:4220/forecast/reportData")
//            HttpRequest httpRequest = HttpRequest.post("http://112.2.27.4/forecast")
                    .header("Content-Type", "application/json");
            httpRequest.setGlobalTimeout(20000);
            response = httpRequest.body(jsonString).execute().body();

            ResponseLXVo responseLXVo = JSONUtil.toBean(response, ResponseLXVo.class);
            if (responseLXVo.getCode().equals("200")) {
                record.setState(StatusEnum.SUCCESS.getSign());
                record.setStateContent(responseLXVo.getMessage());
                log.info("=========={}:{} ==>上送短期原始数据成功  {}条==========", integrationCompany.getName(), station.getStationCode(), dataList.size());
                log.info("{} 原始数据:{}", stationCode, body);
                recordService.save(record);
                forecastDataLxService.saveBatch(forecastDataLxList);
                return true;
            } else {
                log.error("=========={}:{} ==>上送数据失败 失败标识:{} ,失败原因：{} ==========", integrationCompany.getName(), station.getStationCode(), responseLXVo.getCode(), responseLXVo.getMessage());
                log.info("{} 原始数据:{}", stationCode, body);
                record.setState(StatusEnum.FILE_NULL.getSign());
                record.setStateContent(responseLXVo.getMessage());
                recordService.save(record);
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    @Override
    public Boolean pushNWPData(Station station) {
        return null;
    }

    public String addOneDay(String lastForecastDateStr) {
        // 定义日期格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        // 将字符串解析为 LocalDate
        LocalDate lastForecastDate = LocalDate.parse(lastForecastDateStr, formatter);

        // 加一天
        LocalDate nextForecastDate = lastForecastDate.plusDays(1);

        // 格式化为字符串
        String changeGenDate = nextForecastDate.format(formatter);

        String dateTime = changeGenDate + " 00:00:00";

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        long forecast = 0L;
        try {
            forecast = simpleDateFormat.parse(dateTime).getTime();
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }

        return changeGenDate + "--" + forecast;
    }

    /**
     *  生成0.9~1.1之间不包括1的随机数
     * @return
     */
    public Double getRandom(){
        double decision = Math.random();

        double randomNumber;
        if (decision < 0.1) {

            randomNumber = 0.9 + (Math.random() * 0.2);
        } else {
            randomNumber = 0.9 + (Math.random() * 0.1);
        }
        return randomNumber;
    }
}
